{% load static %}

{% include 'bundles/moment-js.html' with locale=True %}
{% include 'bundles/datatables-js.html' %}

<script type="application/javascript" src="{% static 'freight/vendor/datatables/datetime.js' %}"></script>
<script type="application/javascript" src="{% static 'js/filterDropDown/filterDropDown.min.js' %}"></script>
<script type="application/javascript" src="{% static 'freight/js/contracts.js' %}"></script>
