# Generated by Django 2.2.11 on 2020-04-05 16:02

import django.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("freight", "0015_auto_20200130_2328"),
    ]

    operations = [
        migrations.AddField(
            model_name="pricing",
            name="is_default",
            field=models.BooleanField(
                default=False,
                help_text="The default pricing will be preselected in the calculator. Please make sure to only mark one pricing as default.",
            ),
        ),
        migrations.AlterField(
            model_name="location",
            name="category_id",
            field=models.PositiveIntegerField(
                choices=[(3, "station"), (65, "structure"), (0, "(unknown)")],
                default=0,
                help_text="Eve Online category ID",
            ),
        ),
        migrations.AlterField(
            model_name="location",
            name="solar_system_id",
            field=models.PositiveIntegerField(
                blank=True,
                default=None,
                help_text="Eve Online solar system ID",
                null=True,
            ),
        ),
        migrations.AlterField(
            model_name="location",
            name="type_id",
            field=models.PositiveIntegerField(
                blank=True, default=None, help_text="Eve Online type ID", null=True
            ),
        ),
        migrations.AlterField(
            model_name="pricing",
            name="days_to_complete",
            field=models.PositiveIntegerField(
                blank=True,
                default=None,
                help_text="Recommended days for contract completion",
                null=True,
                validators=[django.core.validators.MinValueValidator(1)],
            ),
        ),
        migrations.AlterField(
            model_name="pricing",
            name="days_to_expire",
            field=models.PositiveIntegerField(
                blank=True,
                default=None,
                help_text="Recommended days for contracts to expire",
                null=True,
                validators=[django.core.validators.MinValueValidator(1)],
            ),
        ),
    ]
